# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


class ShareServersNegativeAdminTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ShareServersNegativeAdminTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client

    @decorators.idempotent_id('6d55516f-9018-4372-8310-f725c4562961')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_share_servers_with_member(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.list_share_servers)

    @decorators.idempotent_id('d9021f06-c146-4e76-852e-8a1ebb3fa92e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_show_share_server_with_member(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.show_share_server,
                          'fake_id')

    @decorators.idempotent_id('16b6f911-487e-4c25-9241-75de2dbfc8ff')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_show_share_server_details_with_member(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.show_share_server_details,
                          'fake_id')

    @decorators.idempotent_id('f7580ef6-f7bb-4b52-ba45-82d2d2d66dbe')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_share_server_with_inexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.show_share_server,
                          'fake_id')

    @decorators.idempotent_id('368c5404-483e-4eee-bb80-86206d153ea2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_share_server_details_with_inexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.show_share_server_details,
                          'fake_id')

    @decorators.idempotent_id('664b5201-7ba9-4e33-9534-5307fc003e44')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_wrong_filter_key(self):
        search_opts = {'fake_filter_key': 'ACTIVE'}
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('dcf169c9-1238-40cb-8a5c-ca6aca9d4d6b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_wrong_filter_value(self):
        search_opts = {'host': 123}
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('3e5d6007-5214-4fa2-bd33-dfd3bead67bf')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_fake_status(self):
        search_opts = {"status": data_utils.rand_name("fake_status")}
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('e893b32a-124f-4e5c-a425-58c8a4eac4a5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_fake_host(self):
        search_opts = {"host": data_utils.rand_name("fake_host")}
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('2f1162a8-bb52-4e2a-abc0-68d16f769e4f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_fake_project(self):
        search_opts = {"project_id": data_utils.rand_name("fake_project_id")}
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('ca23f385-56b2-4c02-9797-d88c3b7fb981')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_share_servers_with_fake_share_network(self):
        search_opts = {
            "share_network": data_utils.rand_name("fake_share_network"),
        }
        servers = self.admin_client.list_share_servers(
            search_opts)['share_servers']
        self.assertEqual(0, len(servers))

    @decorators.idempotent_id('0acb9107-18b2-4e9d-8432-37fd0d4c79b3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_server_with_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.delete_share_server,
                          "fake_nonexistent_share_server_id")

    @decorators.idempotent_id('65e12bf7-2ec6-4a5b-971b-b1ecb67b77b7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_server_with_member(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.delete_share_server,
                          "fake_nonexistent_share_server_id")



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


class SecurityServicesMappingTest(base.BaseSharesTest):

    @classmethod
    def resource_setup(cls):
        super(SecurityServicesMappingTest, cls).resource_setup()
        cls.cl = cls.shares_client

    def setUp(self):
        super(SecurityServicesMappingTest, self).setUp()

        # create share network
        data = utils.generate_share_network_data()

        self.sn = self.create_share_network(client=self.cl,
                                            add_security_services=False,
                                            **data)
        self.assertLessEqual(data.items(), self.sn.items())

        # create security service
        data = utils.generate_security_service_data()

        self.ss = self.create_security_service(client=self.cl, **data)
        self.assertLessEqual(data.items(), self.ss.items())

        # Add security service to share network
        self.cl.add_sec_service_to_share_network(self.sn["id"], self.ss["id"])

    @decorators.idempotent_id('e8c5b4d5-7ad2-4aa7-bab0-b454a2e150e9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_map_ss_to_sn_and_list(self):

        # List security services for share network
        ls = self.cl.list_sec_services_for_share_network(
            self.sn["id"])['security_services']
        self.assertEqual(1, len(ls))
        for key in ["status", "id", "name"]:
            self.assertIn(self.ss[key], ls[0][key])

    @decorators.idempotent_id('9dd352b2-6d47-4cab-aa61-52d8081f67a2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_map_ss_to_sn_and_delete(self):

        # Remove security service from share network
        self.cl.remove_sec_service_from_share_network(
            self.sn["id"], self.ss["id"])

    @decorators.idempotent_id('2b0bd5cc-eb35-430f-acfd-f80a2e467667')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_remap_ss_to_sn(self):

        # Remove security service from share network
        self.cl.remove_sec_service_from_share_network(
            self.sn["id"], self.ss["id"])

        # Add security service to share network again
        self.cl.add_sec_service_to_share_network(self.sn["id"], self.ss["id"])



# Copyright 2016 Hitachi Data Systems
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


class BaseShareSnapshotRulesTest(base.BaseSharesMixedTest):

    protocol = ""

    @classmethod
    def resource_setup(cls):
        super(BaseShareSnapshotRulesTest, cls).resource_setup()
        # create share_type
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        cls.snapshot = cls.create_snapshot_wait_for_active(cls.share['id'])

    def _test_create_delete_access_rules(self, access_to):
        # create rule
        rule = self.shares_v2_client.create_snapshot_access_rule(
            self.snapshot['id'], self.access_type,
            access_to)['snapshot_access']

        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, list(rule.keys()))

        waiters.wait_for_resource_status(
            self.shares_v2_client, self.snapshot['id'], 'active',
            resource_name='snapshot_access', rule_id=rule['id'],
            status_attr='state')

        # delete rule and wait for deletion
        self.shares_v2_client.delete_snapshot_access_rule(self.snapshot['id'],
                                                          rule['id'])
        waiters.wait_for_snapshot_access_rule_deletion(
            self.shares_v2_client, self.snapshot['id'], rule['id'])


@ddt.ddt
class ShareSnapshotIpRulesForNFSTest(BaseShareSnapshotRulesTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(ShareSnapshotIpRulesForNFSTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_ip_rules_for_protocols):
            msg = "IP rule tests for %s protocol are disabled." % cls.protocol
            raise cls.skipException(msg)

        utils.check_skip_if_microversion_not_supported('2.32')

    @classmethod
    def resource_setup(cls):
        super(ShareSnapshotIpRulesForNFSTest, cls).resource_setup()
        cls.access_type = "ip"

    @decorators.idempotent_id('bdce2be8-80b9-4f68-bdc0-09a52ba0e6fd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data("1.1.1.1", "1.2.3.4/32")
    def test_create_delete_access_rules(self, access_to):
        self._test_create_delete_access_rules(access_to)


@ddt.ddt
class ShareSnapshotUserRulesForCIFSTest(BaseShareSnapshotRulesTest):
    protocol = "cifs"

    @classmethod
    def skip_checks(cls):
        super(ShareSnapshotUserRulesForCIFSTest, cls).skip_checks()
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_user_rules_for_protocols):
            msg = ("User rule tests for %s protocol are "
                   "disabled." % cls.protocol)
            raise cls.skipException(msg)
        utils.check_skip_if_microversion_not_supported('2.32')

    @classmethod
    def resource_setup(cls):
        super(ShareSnapshotUserRulesForCIFSTest, cls).resource_setup()
        cls.access_type = "user"

    @decorators.idempotent_id('c2625cd2-4dc0-431d-b47b-8f097e22f16d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_delete_access_rules(self):
        access_to = CONF.share.username_for_user_rules
        self._test_create_delete_access_rules(access_to)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacQuotasTests(rbac_base.ShareRbacBaseTests,
                           metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacQuotasTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacQuotasTests, cls).resource_setup()
        cls.quotas = cls.client.show_quotas(cls.client.tenant_id)['quota_set']
        cls.alt_quotas = cls.alt_project_share_v2_client.show_quotas(
            cls.alt_project_share_v2_client.tenant_id)['quota_set']

    @abc.abstractmethod
    def test_default_quotas(self):
        pass

    @abc.abstractmethod
    def test_show_quotas(self):
        pass

    @abc.abstractmethod
    def test_show_quotas_detail(self):
        pass

    @abc.abstractmethod
    def test_update_tenant_quota_shares(self):
        pass

    @abc.abstractmethod
    def test_delete_quotas(self):
        pass


class TestProjectAdminTests(ShareRbacQuotasTests, base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']

    @decorators.idempotent_id('e102292f-93f9-4918-96b2-bb270e29e43e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_default_quotas(self):
        self.do_request(
            'default_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'default_quotas', expected_status=200,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('77c14ee8-9dbc-47dc-a86e-3a26f33beda5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_quotas(self):
        self.do_request(
            'show_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'show_quotas', expected_status=200,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('0bce045c-5575-4301-b526-032812a2e71f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_quotas_detail(self):
        self.do_request(
            'detail_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'detail_quotas', expected_status=200,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('b055f9ea-6176-45f9-a918-d9120912fcf6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_tenant_quota_shares(self):
        self.do_request(
            'update_quotas', expected_status=200,
            tenant_id=self.client.tenant_id,
            shares=int(self.quotas['shares']) + 2)

        self.do_request(
            'update_quotas', expected_status=200,
            tenant_id=self.alt_project_share_v2_client.tenant_id,
            shares=int(self.alt_quotas['shares']) + 2)

    @decorators.idempotent_id('fe9ce5ab-4e93-4bdf-bd2d-d1b35a8b19f8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_quotas(self):
        self.do_request(
            'reset_quotas', expected_status=202,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'reset_quotas', expected_status=202,
            tenant_id=self.alt_project_share_v2_client.tenant_id)


class TestProjectMemberTests(ShareRbacQuotasTests, base.BaseSharesTest):

    credentials = ['project_member', 'project_alt_member']

    @decorators.idempotent_id('a81d40fc-04b2-4535-ad44-c989a51e49b9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_default_quotas(self):
        self.do_request(
            'default_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'default_quotas', expected_status=200,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('52194358-6268-446c-ada4-74fb7e23dbe9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_show_quotas(self):
        self.do_request(
            'show_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'show_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('68b3d3e7-8ebd-4b20-bf56-7b4e4c365eda')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_show_quotas_detail(self):
        self.do_request(
            'detail_quotas', expected_status=200,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'detail_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.alt_project_share_v2_client.tenant_id)

    @decorators.idempotent_id('5a86d62d-5fdf-448e-bd6b-43e26e39201f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_tenant_quota_shares(self):
        self.do_request(
            'update_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.client.tenant_id,
            shares=int(self.quotas['shares']) + 2)

        self.do_request(
            'update_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.alt_project_share_v2_client.tenant_id,
            shares=int(self.alt_quotas['shares']) + 2)

    @decorators.idempotent_id('1928eea7-ca78-4004-8e5f-6d58a446503c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_quotas(self):
        self.do_request(
            'reset_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.client.tenant_id)

        self.do_request(
            'reset_quotas', expected_status=lib_exc.Forbidden,
            tenant_id=self.alt_project_share_v2_client.tenant_id)


class TestProjectReaderTests(TestProjectMemberTests):

    credentials = ['project_reader', 'project_alt_member']

    @decorators.idempotent_id('51ec3c23-8c3b-45ff-9e41-38141ac82145')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_default_quotas(self):
        super(TestProjectReaderTests, self).test_default_quotas()

    @decorators.idempotent_id('48ca6e6b-6ad1-43b6-bdb7-848fe6a4d0fb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_show_quotas(self):
        super(TestProjectReaderTests, self).test_show_quotas()

    @decorators.idempotent_id('0648bf5f-d8c8-4fd4-9713-27e9b5a1cda8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_default_quotas_detail(self):
        super(TestProjectReaderTests, self).test_show_quotas_detail()

    @decorators.idempotent_id('4051f57d-3d79-4007-8b90-b5abf744b4b3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_tenant_quota_shares(self):
        super(TestProjectReaderTests, self).test_update_tenant_quota_shares()

    @decorators.idempotent_id('8185210d-edf4-40e7-840a-484ab21bf7bd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_quotas(self):
        super(TestProjectReaderTests, self).test_delete_quotas()



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


@ddt.ddt
class ManageNFSShareTest(base.BaseSharesAdminTest):
    protocol = 'nfs'

    # NOTE(vponomaryov): be careful running these tests using generic driver
    # because cinder volumes will stay attached to service Nova VM and
    # won't be deleted.

    @classmethod
    def skip_checks(cls):
        super(ManageNFSShareTest, cls).skip_checks()
        if not CONF.share.run_manage_unmanage_tests:
            raise cls.skipException('Manage/unmanage tests are disabled.')
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)
        utils.skip_if_manage_not_supported_for_version()

    @classmethod
    def resource_setup(cls):
        super(ManageNFSShareTest, cls).resource_setup()
        # Create share type
        cls.st_name = data_utils.rand_name("manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }

        cls.st = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)

    def _test_manage(self, is_public=False,
                     version=CONF.share.max_api_microversion,
                     check_manage=False):

        utils.skip_if_manage_not_supported_for_version(version)

        share = self._create_share_for_manage()

        name = "Name for 'managed' share that had ID %s" % share['id']
        description = "Description for 'managed' share"

        # Unmanage share
        self._unmanage_share_and_wait(share)

        if check_manage:
            # After 'unmanage' operation, share instance should be deleted.
            # Assert not related to 'manage' test, but placed here for
            # resource optimization.
            share_instance_list = self.shares_v2_client.list_share_instances(
                )['share_instances']
            share_ids = [si['share_id'] for si in share_instance_list]
            self.assertNotIn(share['id'], share_ids)

        export_path = share['export_locations'][0]
        if utils.is_microversion_ge(version, "2.8"):
            export_path = share['export_locations'][0]['path']

        # Manage share
        manage_params = {
            'service_host': share['host'],
            'export_path': export_path,
            'protocol': share['share_proto'],
            'share_type_id': self.st['id'],
            'name': name,
            'description': description,
            'is_public': is_public,
            'version': version,
        }
        if CONF.share.multitenancy_enabled:
            manage_params['share_server_id'] = share['share_server_id']
        managed_share = self.shares_v2_client.manage_share(
            **manage_params)['share']

        # Add managed share to cleanup queue
        self.method_resources.insert(
            0, {'type': 'share', 'id': managed_share['id'],
                'client': self.shares_client})

        # Wait for success
        waiters.wait_for_resource_status(self.shares_v2_client,
                                         managed_share['id'],
                                         constants.STATUS_AVAILABLE)

        # Verify data of managed share
        self.assertEqual(name, managed_share['name'])
        self.assertEqual(description, managed_share['description'])
        self.assertEqual(share['host'], managed_share['host'])
        self.assertEqual(share['share_proto'], managed_share['share_proto'])

        if utils.is_microversion_ge(version, "2.6"):
            self.assertEqual(self.st['id'],
                             managed_share['share_type'])
        else:
            self.assertEqual(self.st['name'],
                             managed_share['share_type'])

        if utils.is_microversion_ge(version, "2.8"):
            self.assertEqual(is_public, managed_share['is_public'])
        else:
            self.assertFalse(managed_share['is_public'])

        if utils.is_microversion_ge(version, "2.16"):
            self.assertEqual(share['user_id'], managed_share['user_id'])
        else:
            self.assertNotIn('user_id', managed_share)

        # Delete share
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('15b654d0-34ed-4154-9f5f-b96d2e4e9d1c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.5")
    def test_manage_with_os_share_manage_url(self):
        self._test_manage(version="2.5")

    @decorators.idempotent_id('8c0beefb-19da-441e-b73f-d90eb8000ff3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.8")
    def test_manage_with_is_public_True(self):
        self._test_manage(is_public=True, version="2.8")

    @decorators.idempotent_id('da7b7a4f-6693-4460-bdb7-1f8d42032bc6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.16")
    def test_manage_show_user_id(self):
        self._test_manage(version="2.16")

    @decorators.idempotent_id('24ec7840-0174-484f-8b3a-8c6f162bf576')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_manage(self):
        self._test_manage(check_manage=True)


class ManageCIFSShareTest(ManageNFSShareTest):
    protocol = 'cifs'


class ManageGLUSTERFSShareTest(ManageNFSShareTest):
    protocol = 'glusterfs'


class ManageHDFSShareTest(ManageNFSShareTest):
    protocol = 'hdfs'


class ManageCephFSShareTest(ManageNFSShareTest):
    protocol = 'cephfs'


class ManageMapRFSShareTest(ManageNFSShareTest):
    protocol = 'maprfs'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


@ddt.ddt
class ShareInstancesNegativeTest(base.BaseSharesAdminTest):

    @classmethod
    def resource_setup(cls):
        super(ShareInstancesNegativeTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)

    @decorators.idempotent_id('babe885e-a8ab-439d-8b95-e5422983a942')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.34")
    @ddt.data('path', 'id')
    def test_list_share_instances_with_export_location_and_invalid_version(
            self, export_location_type):
        # In API versions <v2.35, querying the share instance API by export
        # location path or ID should have no effect. Those filters were
        # supported from v2.35
        filters = {
            'export_location_' + export_location_type: 'fake',
        }
        share_instances = self.shares_v2_client.list_share_instances(
            params=filters, version="2.34")['share_instances']

        self.assertGreater(len(share_instances), 0)

    @decorators.idempotent_id('ce0d045c-e418-42fa-86e4-ead493fc0663')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.35")
    @ddt.data('path', 'id')
    def test_list_share_instances_with_export_location_not_exist(
            self, export_location_type):
        filters = {
            'export_location_' + export_location_type: 'fake_not_exist',
        }
        share_instances = self.shares_v2_client.list_share_instances(
            params=filters)['share_instances']

        self.assertEqual(0, len(share_instances))



# Copyright 2016 Huawei
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SnapshotInstancesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotInstancesNegativeTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')

        utils.check_skip_if_microversion_not_supported('2.19')

    @classmethod
    def resource_setup(cls):
        super(SnapshotInstancesNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client
        # create share type
        extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id,
                                     client=cls.admin_client)
        cls.snapshot = cls.create_snapshot_wait_for_active(
            cls.share["id"], client=cls.admin_client)

    @decorators.idempotent_id('6e371aac-ff8b-4eac-abc9-b8d777448ff3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshot_instances_with_snapshot_by_non_admin(self):
        self.assertRaises(
            lib_exc.Forbidden,
            self.member_client.list_snapshot_instances,
            snapshot_id=self.snapshot['id'])

    @decorators.idempotent_id('d80331e4-8738-46c7-b726-1e716acef738')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_instance_by_non_admin(self):
        instances = self.admin_client.list_snapshot_instances(
            snapshot_id=self.snapshot['id'])['snapshot_instances']
        self.assertRaises(
            lib_exc.Forbidden,
            self.member_client.get_snapshot_instance,
            instance_id=instances[0]['id'])

    @decorators.idempotent_id('666a6caf-31b0-45d9-808c-e590250cffd4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_snapshot_instance_status_by_non_admin(self):
        instances = self.admin_client.list_snapshot_instances(
            snapshot_id=self.snapshot['id'])['snapshot_instances']
        self.assertRaises(
            lib_exc.Forbidden,
            self.member_client.reset_snapshot_instance_status,
            instances[0]['id'],
            'error')


class SnapshotInstancesNegativeNoResourceTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotInstancesNegativeNoResourceTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')

        utils.check_skip_if_microversion_not_supported('2.19')

    @classmethod
    def resource_setup(cls):
        super(SnapshotInstancesNegativeNoResourceTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client

    @decorators.idempotent_id('abded04d-40c0-4eb9-b2be-58b4efb16244')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_snapshot_instance_with_non_existent_instance(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.get_snapshot_instance,
                          instance_id="nonexistent_instance")

    @decorators.idempotent_id('1609702b-de87-4d12-9a9c-78077d4676f3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_snapshot_instances_by_non_admin(self):
        self.assertRaises(
            lib_exc.Forbidden,
            self.member_client.list_snapshot_instances)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion
RESTRICTED_RULES_VERSION = '2.82'


@ddt.ddt
class ShareIpRulesForNFSNegativeTest(base.BaseSharesMixedTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(ShareIpRulesForNFSNegativeTest, cls).skip_checks()
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_ip_rules_for_protocols):
            msg = "IP rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(ShareIpRulesForNFSNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client

        # create share_type
        extra_specs = None
        if CONF.share.run_snapshot_tests:
            extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap = cls.create_snapshot_wait_for_active(cls.share["id"])

        cls.user_project = cls.os_admin.projects_client.show_project(
            cls.shares_v2_client.project_id)['project']
        cls.new_user = cls.create_user_and_get_client(
            project=cls.user_project)

    @decorators.idempotent_id('16781b45-d2bb-4891-aa97-c28c0769d5bd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('1.2.3.256',
              '1.1.1.-',
              '1.2.3.4/33',
              '1.2.3.*',
              '1.2.3.*/23',
              '1.2.3.1|23',
              '1.2.3.1/-1',
              '1.2.3.1/',
              'ad80::abaa:0:c2:2/-3',
              'AD80:ABAA::|26',
              '2001:DB8:2de:0:0:0:0:e13:200a',
              )
    def test_create_access_rule_ip_with_wrong_target(self, ip_address):
        for client_name in ['shares_client', 'shares_v2_client']:
            self.assertRaises(lib_exc.BadRequest,
                              getattr(self, client_name).create_access_rule,
                              self.share["id"], "ip", ip_address)

    @decorators.idempotent_id('e891deff-23d9-4872-911c-bd9b43dc797f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_with_wrong_level(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"],
                          'ip',
                          '2.2.2.2',
                          'su')

    @decorators.idempotent_id('efd594aa-dd24-427e-acdf-10d124afb572')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('1.0', '2.9', LATEST_MICROVERSION)
    def test_create_duplicate_of_ip_rule(self, version):
        # test data
        access_type = "ip"
        access_to = "1.2.3.4"

        if utils.is_microversion_eq(version, '1.0'):
            client = self.shares_client
        else:
            client = self.shares_v2_client

        # create rule
        self.allow_access(
            self.share["id"], client=client, access_type=access_type,
            access_to=access_to, version=version)

        # try create duplicate of rule
        if utils.is_microversion_eq(version, '1.0'):
            self.assertRaises(lib_exc.BadRequest,
                              self.shares_client.create_access_rule,
                              self.share["id"], access_type, access_to)
        else:
            self.assertRaises(lib_exc.BadRequest,
                              self.shares_v2_client.create_access_rule,
                              self.share["id"], access_type, access_to,
                              version=version)

    @decorators.idempotent_id('63932d1d-a60a-4af7-ba3b-7cf6c68aaee9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data("10.20.30.40", "fd8c:b029:bba6:ac54::1",
              "fd2c:b029:bba6:df54::1/128", "10.10.30.40/32")
    def test_create_duplicate_single_host_rules(self, access_to):
        """Test rules for individual clients with and without max-prefix."""
        if ':' in access_to and utils.is_microversion_lt(
                CONF.share.max_api_microversion, '2.38'):
            reason = ("Skipped. IPv6 rules are accepted from and beyond "
                      "API version 2.38, the configured maximum API version "
                      "is %s" % CONF.share.max_api_microversion)
            raise self.skipException(reason)

        self.allow_access(
            self.share["id"], access_type="ip", access_to=access_to)

        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_access_rule,
                          self.share["id"], "ip", access_to)

        if '/' in access_to:
            access_to = access_to.split("/")[0]
        else:
            access_to = ('%s/32' % access_to if '.' in access_to else
                         '%s/128' % access_to)

        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_access_rule,
                          self.share["id"], "ip", access_to)

    @decorators.idempotent_id('d2856c7d-9417-416d-8d08-e68376ee5b2e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_add_access_rule_on_share_with_no_host(self):
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.protocol)
        extra_specs = self.add_extra_specs_to_dict(
            {"share_backend_name": 'invalid_backend'})
        share_type = self.create_share_type('invalid_backend',
                                            extra_specs=extra_specs,
                                            client=self.admin_client,
                                            cleanup_in_class=False)
        share = self.create_share(share_type_id=share_type['id'],
                                  cleanup_in_class=False,
                                  wait_for_status=False)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], constants.STATUS_ERROR)
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.create_access_rule,
                          share["id"], access_type, access_to)

    @decorators.idempotent_id('478d3c84-b0ea-41c8-a860-e87f182d991c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_deny_access_unrestrict_other_user_rule(self):
        utils.check_skip_if_microversion_not_supported(
            RESTRICTED_RULES_VERSION)

        access_type, access_to = utils.get_access_rule_data_from_config(
            self.protocol)

        # create rule
        rule = self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=access_type, access_to=access_to,
            lock_visibility=True, lock_deletion=True)

        self.assertRaises(
            lib_exc.Forbidden,
            self.new_user.shares_v2_client.delete_access_rule,
            self.share['id'],
            rule['id'],
            unrestrict=True
        )

    @decorators.idempotent_id('c107b0b7-7a3e-4114-af64-ca8fe6e836c9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(True, False)
    def test_deny_access_without_unrestrict_as_owner_user(self, same_user):
        utils.check_skip_if_microversion_not_supported(
            RESTRICTED_RULES_VERSION)
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.protocol)

        # create rule
        rule = self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=access_type, access_to=access_to,
            lock_visibility=True, lock_deletion=True)

        client = (
            self.shares_v2_client
            if same_user else self.new_user.shares_v2_client
        )
        self.assertRaises(
            lib_exc.Forbidden,
            client.delete_access_rule,
            self.share['id'],
            rule['id'])
        self.assertRaises(
            lib_exc.Forbidden,
            client.delete_access_rule,
            self.share['id'],
            rule['id'],
            version='2.81'
        )

    @decorators.idempotent_id('f5b9e7c9-7e6b-4918-a1c4-e03c8d82c46a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_allow_access_multiple_visibility_locks_not_allowed(self):
        utils.check_skip_if_microversion_not_supported(
            RESTRICTED_RULES_VERSION)
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.protocol)

        # create rule
        rule = self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=access_type, access_to=access_to,
            lock_visibility=True, lock_deletion=True)

        self.assertRaises(
            lib_exc.Conflict,
            self.shares_v2_client.create_resource_lock,
            rule['id'],
            "access_rule",
            resource_action="show",
            lock_reason="locked for testing"
        )

        self.assertRaises(
            lib_exc.Conflict,
            self.new_user.shares_v2_client.create_resource_lock,
            rule['id'],
            "access_rule",
            resource_action="show",
            lock_reason="locked for testing"
        )


@ddt.ddt
class ShareIpRulesForCIFSNegativeTest(ShareIpRulesForNFSNegativeTest):
    protocol = "cifs"


@ddt.ddt
class ShareUserRulesForNFSNegativeTest(base.BaseSharesMixedTest):
    protocol = "nfs"

    @classmethod
    def resource_setup(cls):
        super(ShareUserRulesForNFSNegativeTest, cls).resource_setup()
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_user_rules_for_protocols):
            msg = "USER rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)
        # create share type
        extra_specs = None
        if CONF.share.run_snapshot_tests:
            extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap = cls.create_snapshot_wait_for_active(cls.share["id"])

    @decorators.idempotent_id('d6148911-3a0c-4e1f-afdb-fcf203fe4a5b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_wrong_input_2(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "user",
                          "try+")

    @decorators.idempotent_id('a4d8358d-dec0-4c2a-a544-182816a0ba6f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_empty_key(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "user", "")

    @decorators.idempotent_id('f5252e86-4767-48ad-8be5-43e12c93df79')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_too_little_key(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "user", "abc")

    @decorators.idempotent_id('f8f4d3ee-82b8-4d37-917d-a0cd72073df4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_too_big_key(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "user", "a" * 256)

    @decorators.idempotent_id('21724a99-0790-49d5-a069-d1df43782965')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_wrong_input_1(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "user",
                          "try+")

    @decorators.idempotent_id('bc62ce96-36fe-4c9b-b6b9-4d5a661c8035')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_access_rule_user_to_snapshot(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          self.snap["id"],
                          access_type="user",
                          access_to="fakeuser")

    @decorators.idempotent_id('04d5b25f-b335-4574-82b0-f607c8b3bf25')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_user_with_wrong_share_id(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          "wrong_share_id",
                          access_type="user",
                          access_to="fakeuser")

    @decorators.idempotent_id('301bdbd5-4398-4320-b334-7370995369e9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_with_wrong_level(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"],
                          'user',
                          CONF.share.username_for_user_rules,
                          'su')

    @utils.skip_if_microversion_not_supported('2.88')
    @decorators.idempotent_id('d5b1e7c9-7e6b-4918-a1c4-e03c8d82c46a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_access_rule_with_wrong_level(self):
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.protocol)
        if access_type != 'ip':
            msg = "Access rule updates supported only for 'ip' access."
            raise self.skipException(msg)

        rule = self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=access_type, access_to=access_to)

        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.update_access_rule,
            rule['id'],
            access_level='fake_level'
        )


@ddt.ddt
class ShareUserRulesForCIFSNegativeTest(ShareUserRulesForNFSNegativeTest):
    protocol = "cifs"


@ddt.ddt
class ShareCertRulesForGLUSTERFSNegativeTest(base.BaseSharesMixedTest):
    protocol = "glusterfs"

    @classmethod
    def resource_setup(cls):
        super(ShareCertRulesForGLUSTERFSNegativeTest, cls).resource_setup()
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_cert_rules_for_protocols):
            msg = "CERT rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)
        # create share type
        extra_specs = None
        if CONF.share.run_snapshot_tests:
            extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap = cls.create_snapshot_wait_for_active(cls.share["id"])

    @decorators.idempotent_id('a16d53d5-50d4-4015-912f-2850c5d62690')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_cert_with_empty_common_name(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "cert", "")

    @decorators.idempotent_id('7b5383d8-5bcd-47aa-955b-ed3757a5bdb4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_cert_with_whitespace_common_name(self,
                                                                 client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "cert", " ")

    @decorators.idempotent_id('1c25c134-92b4-4875-a061-88d394e28bcc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_cert_with_too_big_common_name(self,
                                                              client_name):
        # common name cannot be more than 64 characters long
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "cert", "a" * 65)

    @decorators.idempotent_id('dd85d5cd-aa83-4f44-8572-bd7e68a84fb2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_cert_to_snapshot(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          self.snap["id"],
                          access_type="cert",
                          access_to="fakeclient1.com")

    @decorators.idempotent_id('eb47a511-7688-4689-a2ad-54ba85b39b07')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_cert_with_wrong_share_id(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          "wrong_share_id",
                          access_type="cert",
                          access_to="fakeclient2.com")


@ddt.ddt
class ShareCephxRulesForCephFSNegativeTest(base.BaseSharesMixedTest):
    protocol = "cephfs"

    @classmethod
    def resource_setup(cls):
        super(ShareCephxRulesForCephFSNegativeTest, cls).resource_setup()
        if not (cls.protocol in CONF.share.enable_protocols and
                cls.protocol in CONF.share.enable_cephx_rules_for_protocols):
            msg = ("CEPHX rule tests for %s protocol are disabled" %
                   cls.protocol)
            raise cls.skipException(msg)
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        cls.access_type = "cephx"
        cls.access_to = "david"

    @decorators.idempotent_id('7b33c073-353e-4952-97dc-c3948a3cd037')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('jane.doe', "bj\u00F6rn")
    def test_create_access_rule_cephx_with_invalid_cephx_id(self, access_to):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_access_rule,
                          self.share["id"], self.access_type, access_to)

    @decorators.idempotent_id('16b7d848-2f7c-4709-85a3-2dfb4576cc59')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_access_rule_cephx_admin_user(self):
        """CVE-2020-27781 - using admin in cephx rule must be disallowed"""

        self.assertRaises(share_exceptions.AccessRuleBuildErrorException,
                          self.allow_access,
                          self.share["id"],
                          access_type=self.access_type,
                          access_to='admin')

    @decorators.idempotent_id('dd8be44c-c7e8-42fe-b81c-095a1c66730c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_access_rule_cephx_with_wrong_level(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_access_rule,
                          self.share["id"], self.access_type, self.access_to,
                          access_level="su")

    @decorators.idempotent_id('4ffed391-d7cc-481b-bb74-9f3406ddd75f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_different_tenants_cannot_use_same_cephx_id(self):
        scheduler_hint = {"same_host": "%s" % self.share["id"]}

        # Grant access to the share
        self.allow_access(self.share['id'], access_to=self.access_to)
        # Create second share by the new user
        share2 = self.create_share(client=self.admin_shares_v2_client,
                                   share_protocol=self.protocol,
                                   share_type_id=self.share_type_id,
                                   scheduler_hints=scheduler_hint)

        # Try grant access to the second share using the same cephx id as used
        # on the first share.
        # Rule must be set to "error" status.
        self.allow_access(share2['id'], client=self.admin_shares_v2_client,
                          access_to=self.access_to, status='error',
                          raise_rule_in_error_state=False)

        share_alt_updated = self.admin_shares_v2_client.get_share(
            share2['id'])['share']
        self.assertEqual('error', share_alt_updated['access_rules_status'])

    @decorators.idempotent_id('1a9f46f0-d4e1-40ac-8726-aedd0320d583')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_can_apply_new_cephx_rules_when_one_is_in_error_state(self):
        # Create share on "primary" tenant
        share_primary = self.create_share(
            share_type_id=self.share_type_id)
        # Add access rule to "Joe" by "primary" user
        self.allow_access(share_primary['id'], access_to='Joe')

        scheduler_hint = {"same_host": "%s" % share_primary["id"]}
        # Create share on "admin" tenant
        share_adm = self.create_share(
            client=self.admin_shares_v2_client,
            share_type_id=self.share_type_id,
            scheduler_hints=scheduler_hint)
        # Add access rule to "Joe" by "admin" user.
        # Rule must be set to "error" status.
        rule1 = self.allow_access(share_adm['id'],
                                  client=self.admin_shares_v2_client,
                                  access_to='Joe',
                                  status='error',
                                  raise_rule_in_error_state=False,
                                  cleanup=False)

        # Share's "access_rules_status" must be in "error" status
        share_adm_updated = self.admin_shares_v2_client.get_share(
            share_adm['id'])['share']
        self.assertEqual('error', share_adm_updated['access_rules_status'])

        # Add second access rule to different client by "admin" user.
        self.allow_access(share_adm['id'], client=self.admin_shares_v2_client)

        # Check share's access_rules_status has transitioned to "active" status
        self.admin_shares_v2_client.delete_access_rule(
            share_adm['id'], rule1['id'])
        waiters.wait_for_resource_status(
            self.admin_shares_v2_client, share_adm['id'], 'active',
            status_attr='access_rules_status')


@ddt.ddt
class ShareRulesNegativeTest(base.BaseSharesMixedTest):
    # Tests independent from rule type and share protocol

    @classmethod
    def skip_checks(cls):
        super(ShareRulesNegativeTest, cls).skip_checks()
        if not (any(p in CONF.share.enable_ip_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_user_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cert_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cephx_rules_for_protocols
                    for p in cls.protocols)):
            cls.message = "Rule tests are disabled"
            raise cls.skipException(cls.message)

    @classmethod
    def resource_setup(cls):
        super(ShareRulesNegativeTest, cls).resource_setup()
        # create share type
        extra_specs = None
        if CONF.share.run_snapshot_tests:
            extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)
        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap = cls.create_snapshot_wait_for_active(cls.share["id"])

    @decorators.idempotent_id('84da9231-5c4b-4615-8500-8fc6d30ff7ea')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_delete_access_rule_with_wrong_id(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).delete_access_rule,
                          self.share["id"], "wrong_rule_id")

    @decorators.idempotent_id('13f9329f-12db-467d-9268-a9cca75997d9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_ip_with_wrong_type(self, client_name):
        self.assertRaises(lib_exc.BadRequest,
                          getattr(self, client_name).create_access_rule,
                          self.share["id"], "wrong_type", "1.2.3.4")

    @decorators.idempotent_id('fd6ede10-97d6-4ee8-a661-c516b7421c91')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('shares_client', 'shares_v2_client')
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_access_rule_ip_to_snapshot(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          self.snap["id"])


@ddt.ddt
class ShareRulesAPIOnlyNegativeTest(base.BaseSharesTest):

    @decorators.idempotent_id('01279461-3ccc-49b2-a615-d7984dd0db8c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_create_access_rule_ip_with_wrong_share_id(self, client_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self, client_name).create_access_rule,
                          "wrong_share_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class ShareNetworkSubnetMetadataTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareNetworkSubnetMetadataTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.78")

    @classmethod
    def resource_setup(cls):
        super(ShareNetworkSubnetMetadataTest, cls).resource_setup()
        # create share_network and subnet
        cls.share_network = cls.create_share_network(cleanup_in_class=True)
        az = cls.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        cls.az_name = az['name']

        cls.data = utils.generate_subnet_data()
        cls.data['share_network_id'] = cls.share_network['id']
        cls.data['availability_zone'] = cls.az_name

        cls.subnet = cls.create_share_network_subnet(cleanup_in_class=True,
                                                     **cls.data)

    def _verify_subnet_metadata(self, subnet, md):
        # get metadata of share-network-subnet
        metadata = self.shares_v2_client.get_metadata(
            subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=subnet['share_network_id'])['metadata']

        # verify metadata
        self.assertEqual(md, metadata)

        # verify metadata items
        for key in md:
            get_value = self.shares_v2_client.get_metadata_item(
                subnet['id'], key, resource="subnet",
                parent_resource="share-networks",
                parent_id=subnet['share_network_id'])['meta']
            self.assertEqual(md[key], get_value[key])

    @decorators.idempotent_id('260744c2-c062-4ce3-a57e-cce475650e7b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_in_subnet_creation(self):
        share_network = self.create_share_network()
        az = self.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        az_name = az['name']

        # Generate subnet data
        data = utils.generate_subnet_data()
        data['share_network_id'] = share_network['id']
        data['availability_zone'] = az_name

        md = {"key1": "value1", "key2": "value2", }

        # create network subnet with metadata
        subnet = self.create_share_network_subnet(metadata=md,
                                                  cleanup_in_class=False,
                                                  **data)

        # verify metadata
        self._verify_subnet_metadata(subnet, md)

        # Delete the subnets
        self.shares_v2_client.delete_subnet(share_network['id'], subnet['id'])

    @decorators.idempotent_id('ec5c02e9-fcee-4890-87bc-7937a247afe9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_get_delete_metadata(self):
        md = {"key3": "value3", "key4": "value4", "key.5.1": "value.5"}

        # create subnet
        subnet = self.create_share_network_subnet(**self.data)

        # set metadata
        self.shares_v2_client.set_metadata(
            subnet['id'], md, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # verify metadata
        self._verify_subnet_metadata(subnet, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(
                subnet['id'], key, resource="subnet",
                parent_resource="share-networks",
                parent_id=self.share_network['id'])

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']
        self.assertEmpty(get_metadata)

        # Delete the subnet
        self.shares_v2_client.delete_subnet(self.share_network['id'],
                                            subnet['id'])

    @decorators.idempotent_id('9ff9c3b4-9bd0-4e8a-a317-726cab640a67')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_not_delete_pre_metadata(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key11": "value11", "key12": "value12", }

        # create subnet
        subnet = self.create_share_network_subnet(**self.data)

        # set metadata
        self.shares_v2_client.set_metadata(
            subnet['id'], md1, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # verify metadata
        self._verify_subnet_metadata(subnet, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(
            subnet['id'], md2, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        md1.update(md2)
        md = md1

        # verify metadata
        self._verify_subnet_metadata(subnet, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(
                subnet['id'], key, resource="subnet",
                parent_resource="share-networks",
                parent_id=self.share_network['id'])

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']
        self.assertEmpty(get_metadata)

        # Delete the subnet
        self.shares_v2_client.delete_subnet(self.share_network['id'],
                                            subnet['id'])

    @decorators.idempotent_id('1973bcb0-93a5-49a5-84fb-a03f6f6ff43b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_key_already_exist(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key9": "value13", "key11": "value11", }

        # create subnet
        subnet = self.create_share_network_subnet(**self.data)

        # set metadata
        self.shares_v2_client.set_metadata(
            subnet['id'], md1, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # verify metadata
        self._verify_subnet_metadata(subnet, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(
            subnet['id'], md2, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # verify metadata
        md1.update(md2)
        self._verify_subnet_metadata(subnet, md1)

        # delete metadata
        for key in md1.keys():
            self.shares_v2_client.delete_metadata(
                subnet['id'], key, resource="subnet",
                parent_resource="share-networks",
                parent_id=self.share_network['id'])

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']
        self.assertEmpty(get_metadata)

        # Delete the subnets
        self.shares_v2_client.delete_subnet(self.share_network['id'],
                                            subnet['id'])

    @decorators.idempotent_id('d37ea163-7215-4c35-996a-1bc165e554de')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_and_update_metadata_by_key(self):
        md1 = {"key5": "value5", "key6": "value6", }
        md2 = {"key7": "value7", "key8": "value8", }

        # create subnet
        subnet = self.create_share_network_subnet(**self.data)

        # set metadata
        self.shares_v2_client.set_metadata(
            subnet['id'], md1, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # update metadata
        self.shares_v2_client.update_all_metadata(
            subnet['id'], md2, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        # verify metadata
        self._verify_subnet_metadata(subnet, md2)

        # Delete the subnets
        self.shares_v2_client.delete_subnet(self.share_network['id'],
                                            subnet['id'])

    @decorators.idempotent_id('20b96e7e-33cf-41f2-8f00-357170fa27f9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.set_metadata(self.subnet['id'],
                                           data, resource="subnet",
                                           parent_resource="share-networks",
                                           parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']
        self.assertEqual(data['k'], body_get.get('k'))

    @decorators.idempotent_id('b7933b35-04e7-4487-8e6a-730f7261a736')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.set_metadata(self.subnet['id'],
                                           data, resource="subnet",
                                           parent_resource="share-networks",
                                           parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertIn(max_key, body_get)
        self.assertEqual(data[max_key], body_get.get(max_key))

    @decorators.idempotent_id('469cd8ca-5846-4ff7-9a94-1087fc369e0f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.set_metadata(self.subnet['id'],
                                           data, resource="subnet",
                                           parent_resource="share-networks",
                                           parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('5aa3b4a2-79d3-4fa4-b923-eb120bbe2f29')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_set_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.set_metadata(self.subnet['id'],
                                           data, resource="subnet",
                                           parent_resource="share-networks",
                                           parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('336970cb-eb04-4d08-9941-44c1267c1d5a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_upd_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.update_all_metadata(
            self.subnet['id'], data, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data, body_get)

    @decorators.idempotent_id('93ec3f28-4cd7-48da-8418-e945b9ec4530')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_upd_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.update_all_metadata(
            self.subnet['id'], data, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data, body_get)

    @decorators.idempotent_id('f9372c69-559a-47b8-b6a2-0b2876d7a985')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_upd_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.update_all_metadata(
            self.subnet['id'], data, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data, body_get)

    @decorators.idempotent_id('fdb6696f-910a-41a7-aab3-79addad86936')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_upd_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.update_all_metadata(
            self.subnet['id'], data, resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])

        body_get = self.shares_v2_client.get_metadata(
            self.subnet['id'], resource="subnet",
            parent_resource="share-networks",
            parent_id=self.share_network['id'])['metadata']

        self.assertEqual(data, body_get)



# Copyright 2019 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ShareNetworkSubnetsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareNetworkSubnetsTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.51")

    @classmethod
    def resource_setup(cls):
        super(ShareNetworkSubnetsTest, cls).resource_setup()
        # create share_type
        cls.extra_specs = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        cls.share_type = cls.create_share_type(extra_specs=cls.extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share_network
        cls.share_network = cls.create_share_network()
        cls.share_network_id = cls.share_network['id']

    @decorators.idempotent_id('3e1e4da7-049f-404e-8673-142695a9a785')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_delete_subnet(self):
        share_network = self.shares_v2_client.create_share_network(
            )['share_network']
        share_network = self.shares_v2_client.get_share_network(
            share_network['id']
        )['share_network']
        default_subnet = share_network['share_network_subnets'][0]

        az = self.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        az_name = az['name']

        # Generate subnet data
        data = utils.generate_subnet_data()
        data['share_network_id'] = share_network['id']
        data['availability_zone'] = az_name

        # create a new share network subnet
        created = self.create_share_network_subnet(**data)
        data['share_network_name'] = share_network['name']
        # verify keys
        keys = [
            "share_network_name", "id", "network_type", "cidr",
            "ip_version", "neutron_net_id", "neutron_subnet_id", "created_at",
            "updated_at", "segmentation_id", "availability_zone", "gateway",
            "share_network_id", "mtu"
        ]
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.78'):
            keys.extend(['metadata'])

        # Default subnet was created during share network creation
        self.assertIsNone(default_subnet['availability_zone'])
        # Match new subnet content
        self.assertLessEqual(data.items(), created.items())

        self.assertEqual(sorted(keys), sorted(list(created.keys())))

        # Delete the subnets
        self.shares_v2_client.delete_subnet(share_network['id'], created['id'])

    @decorators.idempotent_id('51c6836a-c6d2-4b80-a992-cf91f9a4332b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_share_network_subnet(self):
        share_network = self.create_share_network()
        az = self.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        az_name = az['name']

        # Generate subnet data
        data = utils.generate_subnet_data()
        data['share_network_id'] = share_network['id']
        data['availability_zone'] = az_name

        # Create the share network subnet
        created = self.create_share_network_subnet(**data)

        # Shows the share network subnet
        shown = self.shares_v2_client.get_subnet(
            created['id'], share_network['id'])['share_network_subnet']

        # Asserts
        self.assertLessEqual(data.items(), shown.items())

        # Deletes the created subnet
        self.shares_v2_client.delete_subnet(share_network['id'],
                                            created['id'])

    @decorators.idempotent_id('89ed6115-eb1d-4a7e-a0a3-9b4a239fadc1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    def test_create_share_on_subnet_with_availability_zone(self):
        compatible_azs = self.get_availability_zones_matching_share_type(
            self.share_type)
        if len(compatible_azs) < 2:
            msg = ("This test needs at least two compatible storage "
                   "availability zones.")
            raise self.skipException(msg)
        check_multiple_subnet = utils.is_microversion_ge(
            CONF.share.max_api_microversion, '2.70')

        original_share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id
        )['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(original_share_network))
        share_network = self.create_share_network(
            neutron_net_id=share_net_info['neutron_net_id'],
            neutron_subnet_id=share_net_info['neutron_subnet_id'],
        )
        share_network = self.shares_v2_client.get_share_network(
            share_network['id']
        )['share_network']
        default_subnet = share_network['share_network_subnets'][0]
        availability_zone = compatible_azs[0]

        data = {
            "neutron_net_id": share_net_info['neutron_net_id'],
            "neutron_subnet_id": share_net_info['neutron_subnet_id'],
            'share_network_id': share_network['id'],
            'availability_zone': availability_zone,
        }
        # Create a new share network subnet
        subnet = self.create_share_network_subnet(**data)

        # Create a new share in the select availability zone
        # The 'status' of the share returned by the create API must be
        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=share_network['id'],
            availability_zone=availability_zone)
        # Set and have value either 'creating' or
        # 'available' (if share creation is really fast as in
        # case of Dummy driver).
        self.assertIn(share['status'], ('creating', 'available'))

        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        # Default subnet was created during share network creation
        self.assertIsNone(default_subnet['availability_zone'])
        # Match new subnet content
        self.assertLessEqual(data.items(), subnet.items())
        # Match share server subnet
        if check_multiple_subnet:
            self.assertIn(subnet['id'],
                          share_server['share_network_subnet_ids'])
        else:
            self.assertIn(subnet['id'],
                          share_server['share_network_subnet_id'])
        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete the subnets
        self.shares_v2_client.delete_subnet(share_network['id'], subnet['id'])

    @decorators.idempotent_id('043fbe02-466d-4344-8e2f-f02cb65132cb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @ddt.data(True, False)
    def test_create_share_on_share_network_with_multiple_subnets(
            self, create_share_with_az):
        compatible_azs = self.get_availability_zones_matching_share_type(
            self.share_type)
        if len(compatible_azs) < 2:
            msg = ("This test needs at least two compatible storage "
                   "availability zones.")
            raise self.skipException(msg)
        check_multiple_subnet = utils.is_microversion_ge(
            CONF.share.max_api_microversion, '2.70')

        original_share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id)['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(original_share_network))
        share_network = self.create_share_network(
            neutron_net_id=share_net_info['neutron_net_id'],
            neutron_subnet_id=share_net_info['neutron_subnet_id'],
        )
        share_network = self.shares_v2_client.get_share_network(
            share_network['id']
        )['share_network']
        default_subnet = share_network['share_network_subnets'][0]
        # Save one availability zone to remain associated with default subnet
        destination_az = compatible_azs.pop()
        if not create_share_with_az:
            destination_az = None

        new_subnets = []
        data = {
            "neutron_net_id": share_net_info['neutron_net_id'],
            "neutron_subnet_id": share_net_info['neutron_subnet_id'],
            'share_network_id': share_network['id'],
        }
        for availability_zone in compatible_azs:
            # update availability zone
            data['availability_zone'] = availability_zone
            # create a new share network subnet
            subnet = self.create_share_network_subnet(**data)
            new_subnets.append(subnet)

        # Create a new share in the selected availability zone
        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=share_network['id'],
            availability_zone=destination_az)
        # The 'status' of the share returned by the create API must be
        # set and have value either 'creating' or 'available' (if share
        # creation is really fast as in case of Dummy driver).
        self.assertIn(share['status'], ('creating', 'available'))

        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']
        # If no availability zone was provided during share creation, it is
        # expected that the Scheduler selects one of the compatible backends to
        # place the share. The destination availability zone may or may not
        # have an specific share network subnet.
        expected_subnet_id = (
            next((subnet['id'] for subnet in new_subnets
                 if subnet['availability_zone'] == share['availability_zone']),
                 default_subnet['id']))
        # Default subnet was created during share network creation
        self.assertIsNone(default_subnet['availability_zone'])
        # Match share server subnet
        if not check_multiple_subnet:
            self.assertEqual(
                expected_subnet_id, share_server['share_network_subnet_id'])
        else:
            self.assertIn(
                expected_subnet_id, share_server['share_network_subnet_ids'])
        if create_share_with_az:
            self.assertEqual(destination_az,
                             share['availability_zone'])
        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete the subnets
        for subnet in new_subnets:
            self.shares_v2_client.delete_subnet(share_network['id'],
                                                subnet['id'])



# Copyright 2015 Yogesh Kshirsagar
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
_MIN_SUPPORTED_MICROVERSION = '2.11'


class ReplicationNegativeBase(base.BaseSharesMixedTest):
    @classmethod
    def skip_checks(cls):
        super(ReplicationNegativeBase, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ReplicationNegativeBase, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )

        # create share type
        extra_specs = {"replication_type": cls.replication_type}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.sn_id = cls.share_network['id']
        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type, client=cls.admin_client)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

        # create share with above share_type
        cls.share1, cls.instance_id1 = cls._create_share_get_instance()

    @classmethod
    def _create_share_get_instance(cls, share_network_id=None):
        sn_id = share_network_id if share_network_id else cls.sn_id
        share = cls.create_share(share_type_id=cls.share_type_id,
                                 availability_zone=cls.share_zone,
                                 share_network_id=sn_id)
        share_instances = cls.admin_client.get_instances_of_share(
            share["id"], version=_MIN_SUPPORTED_MICROVERSION
        )['share_instances']
        instance_id = share_instances[0]["id"]
        return share, instance_id


@ddt.ddt
class ReplicationNegativeTest(ReplicationNegativeBase):

    def _is_replication_type_promotable(self):
        if (self.replication_type
                not in constants.REPLICATION_PROMOTION_CHOICES):
            msg = "Option backend_replication_type should be one of (%s)!"
            raise self.skipException(
                msg % ','.join(constants.REPLICATION_PROMOTION_CHOICES))

    @decorators.idempotent_id('90c5947d-b857-406c-b7dc-7010cbee6482')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_add_replica_to_share_with_no_replication_share_type(self):
        # Create share without replication type
        share_type = self.create_share_type(
            data_utils.rand_name(constants.TEMPEST_MANILA_PREFIX),
            extra_specs=self.add_extra_specs_to_dict(),
            client=self.admin_client)
        share = self.create_share(share_type_id=share_type["id"],
                                  share_network_id=self.sn_id)
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_replica,
                          share['id'],
                          self.replica_zone)

    @decorators.idempotent_id('d14ba73e-4f13-451e-86d5-ffeb188e51b7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_add_replica_to_share_with_error_state(self):
        # Set "error" state
        self.admin_client.reset_state(
            self.share1['id'], constants.STATUS_ERROR)
        self.addCleanup(self.admin_client.reset_state,
                        self.share1['id'],
                        constants.STATUS_AVAILABLE)
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_replica,
                          self.share1['id'],
                          self.replica_zone)

    @decorators.idempotent_id('94b41c28-e7a9-4f19-9315-4a57210141af')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_delete_last_active_replica(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.delete_share_replica,
                          self.instance_id1)

    @decorators.idempotent_id('6fbf984d-4efa-41d7-88bf-11ef2f078c02')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_delete_share_having_replica(self):
        self.create_share_replica(self.share1["id"], self.replica_zone,
                                  cleanup_in_class=False)
        self.assertRaises(lib_exc.Conflict,
                          self.shares_v2_client.delete_share,
                          self.share1["id"])

    @decorators.unstable_test(bug='1631314')
    @decorators.idempotent_id('b9c2e57b-f1ae-475c-9d0b-df75dbe93b61')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_promote_out_of_sync_share_replica(self):
        # Test promoting an out_of_sync share_replica to active state
        self._is_replication_type_promotable()
        share, instance_id = self._create_share_get_instance()
        replica = self.create_share_replica(share["id"], self.replica_zone,
                                            cleanup_in_class=False)
        # Set replica state to out of sync
        self.admin_client.reset_share_replica_state(
            replica['id'], constants.REPLICATION_STATE_OUT_OF_SYNC)
        waiters.wait_for_resource_status(
            self.shares_v2_client, replica['id'],
            constants.REPLICATION_STATE_OUT_OF_SYNC,
            resource_name='share_replica', status_attr='replica_state')
        # Try promoting the first out_of_sync replica to active state
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.promote_share_replica,
                          replica['id'])

    @decorators.idempotent_id('e525c3d0-9f45-489f-8e78-ed9e3ac540f7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_promote_active_share_replica(self):
        # Test promote active share_replica
        self._is_replication_type_promotable()

        # Try promoting the active replica
        self.shares_v2_client.promote_share_replica(self.instance_id1,
                                                    expected_status=200)

    @decorators.idempotent_id('37dc3f7e-5a36-4020-ae1b-3c5e6771f024')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_promote_share_replica_for_writable_share_type(self):
        # Test promote active share_replica for writable share
        if self.replication_type != "writable":
            raise self.skipException("Option backend_replication_type "
                                     "should be writable!")
        share, instance_id = self._create_share_get_instance()
        replica = self.create_share_replica(share["id"], self.replica_zone,
                                            cleanup_in_class=False)
        # By default, 'writable' replica is expected to be in active state
        waiters.wait_for_resource_status(
            self.shares_v2_client, replica["id"],
            constants.REPLICATION_STATE_ACTIVE, resource_name='share_replica',
            status_attr='replica_state')

        # Try promoting the replica
        self.shares_v2_client.promote_share_replica(replica['id'])

    @decorators.idempotent_id('600a13d2-5cf0-482e-97af-9f598b55a407')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('2.11', '2.73')
    def test_add_access_rule_share_replica_error_status(self, version):
        """From 2.74, we can add rules even if replicas are in error state."""
        utils.check_skip_if_microversion_not_supported(version)
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.shares_v2_client.share_protocol)
        # Create the replica
        share_replica = self.create_share_replica(self.share1["id"],
                                                  self.replica_zone,
                                                  cleanup_in_class=False)
        # Reset the replica status to error
        self.admin_client.reset_share_replica_status(
            share_replica['id'], constants.STATUS_ERROR)

        if utils.is_microversion_lt(version, '2.74'):
            # Verify access rule cannot be added
            self.assertRaises(lib_exc.BadRequest,
                              self.shares_v2_client.create_access_rule,
                              self.share1["id"], access_type, access_to, 'ro',
                              version)

    @decorators.idempotent_id('91b93b71-4048-412b-bb42-0fe88edfb987')
    @testtools.skipUnless(CONF.share.run_host_assisted_migration_tests or
                          CONF.share.run_driver_assisted_migration_tests,
                          "Share migration tests are disabled.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migration_of_replicated_share(self):
        pools = self.admin_client.list_pools(detail=True)['pools']
        hosts = [p['name'] for p in pools]
        self.create_share_replica(self.share1["id"], self.replica_zone,
                                  cleanup_in_class=False)
        share_host = self.admin_client.get_share(
            self.share1['id'])['share']['host']

        for host in hosts:
            if host != share_host:
                dest_host = host
                break

        self.assertRaises(
            lib_exc.Conflict, self.admin_client.migrate_share,
            self.share1['id'], dest_host)

    @decorators.idempotent_id('bf01bcfc-57cb-4e56-957f-8aa9f1b9be1b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.48")
    def test_try_add_replica_share_type_azs_unsupported_az(self):
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type['id'], 'availability_zones', 'non-existent az')
        self.addCleanup(
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            self.share_type['id'], 'availability_zones')
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_replica,
                          self.share1['id'],
                          self.replica_zone)

    @decorators.idempotent_id('97fc5ab3-c6e2-4da4-ae45-08d458a5ba0c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @utils.skip_if_microversion_not_supported("2.51")
    def test_try_add_replica_nonexistent_subnet(self):
        # Create a new share network only for a specific az
        data = utils.generate_share_network_data()
        subnet = utils.share_network_get_default_subnet(self.share_network)
        data['neutron_net_id'] = subnet['neutron_net_id']
        data['neutron_subnet_id'] = subnet['neutron_subnet_id']
        data['availability_zone'] = self.share_zone
        share_net = self.shares_v2_client.create_share_network(
            **data)['share_network']
        share, instance_id = self._create_share_get_instance(
            share_network_id=share_net['id'])

        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_replica,
                          share['id'],
                          self.replica_zone)


class ReplicationAPIOnlyNegativeTest(base.BaseSharesTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationAPIOnlyNegativeTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @decorators.idempotent_id('72395c9b-4432-4a8b-84b4-60303e6bc962')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_replica_by_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_share_replica,
                          data_utils.rand_uuid())

    @decorators.idempotent_id('41d45772-e021-4404-a991-85f7a6e6431b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_replica_by_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_share_replica,
                          data_utils.rand_uuid())



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_config import types

service_option = cfg.BoolOpt("manila",
                             default=True,
                             help="Whether or not manila is expected to be "
                                  "available")

manila_scope_enforcement = cfg.BoolOpt('manila',
                                       default=False,
                                       help="Does the Share service API "
                                            "policies enforce scope? "
                                            "This configuration value should "
                                            "be same as manila.conf: "
                                            "[oslo_policy].enforce_scope "
                                            "option.")

share_group = cfg.OptGroup(name="share", title="Share Service Options")

ShareGroup = [
    cfg.StrOpt("min_api_microversion",
               default="2.0",
               help="The minimum api microversion is configured to be the "
                    "value of the minimum microversion supported by Manila. "
                    "This value is only used to validate the versions "
                    "response from Manila."),
    cfg.StrOpt("max_api_microversion",
               default="2.88",
               help="The maximum api microversion is configured to be the "
                    "value of the latest microversion supported by Manila."),
    cfg.StrOpt("region",
               default="",
               help="The share region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt("catalog_type",
               default="share",
               help="Catalog type of the Share service."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the share service."),
    cfg.BoolOpt("multitenancy_enabled",
                default=True,
                help="This option used to determine backend driver type, "
                     "multitenant driver uses share-networks, but "
                     "single-tenant doesn't."),
    cfg.BoolOpt("create_networks_when_multitenancy_enabled",
                default=True,
                help="This option is used only when other "
                     "'multitenancy_enabled' option is set to 'True'. "
                     "If this one is set to True, then tempest will create "
                     "neutron networks for each new manila share-network "
                     "it creates. Else it will use manila share-networks with "
                     "empty values (case of StandAloneNetworkPlugin and "
                     "NeutronSingleNetworkPlugin)."),
    cfg.ListOpt("enable_protocols",
                default=["nfs", "cifs"],
                help="First value of list is protocol by default, "
                     "items of list show enabled protocols at all."),
    cfg.ListOpt("enable_ip_rules_for_protocols",
                default=["nfs", "cifs", ],
                help="Selection of protocols, that should "
                     "be covered with ip rule tests"),
    cfg.ListOpt("enable_user_rules_for_protocols",
                default=[],
                help="Selection of protocols, that should "
                     "be covered with user rule tests"),
    cfg.ListOpt("enable_cert_rules_for_protocols",
                default=["glusterfs", ],
                help="Protocols that should be covered with cert rule tests."),
    cfg.ListOpt("enable_cephx_rules_for_protocols",
                default=["cephfs", ],
                help="Protocols to be covered with cephx rule tests."),
    cfg.StrOpt("username_for_user_rules",
               default="Administrator",
               help="Username, that will be used in user tests."),
    cfg.StrOpt("override_ip_for_nfs_access",
               help="Forces access rules to be as specified on NFS scenario"
                    " tests. This can used for working around multiple "
                    "NATs between the VMs and the storage controller."),
    cfg.StrOpt("storage_network",
               help="Name or UUID of a neutron network that is used to access "
                    "shared file systems over. If specified, test virtual "
                    "machines are created with two NICs, the primary NIC is "
                    "attached to the private project network and the "
                    "secondary NIC is attached to the specified storage "
                    "network. If using NFS, access control is done with the "
                    "help of the IP address assigned to the virtual machine's "
                    "storage network NIC."),
    cfg.ListOpt("enable_ro_access_level_for_protocols",
                default=["nfs", ],
                help="List of protocols to run tests with ro access level."),
    cfg.ListOpt("nfs_versions",
                default=["4", ],
                help="Specifies the NFS protocol version to use when mounting "
                     "an NFS share. Set to '3' for NFSv3, and '4' or '4.1' "
                     "for NFSv4. Leave it blank to use the default version."),


    # Capabilities
    cfg.StrOpt("capability_storage_protocol",
               deprecated_name="storage_protocol",
               default="NFS_CIFS",
               help="Backend protocol to target when creating volume types."),
    cfg.BoolOpt("capability_snapshot_support",
                help="Defines extra spec that satisfies specific back end "
                     "capability called 'snapshot_support' and will be used "
                     "for setting up custom share type. Defaults to value of "
                     "other config option 'run_snapshot_tests'."),
    cfg.BoolOpt("capability_create_share_from_snapshot_support",
                help="Defines extra spec that satisfies specific back end "
                     "capability called 'create_share_from_snapshot_support' "
                     "and will be used for setting up a custom share type. "
                     "Defaults to the value of run_snapshot_tests. Set it to "
                     "False if the driver being tested does not support "
                     "creating shares from snapshots."),
    cfg.BoolOpt("capability_revert_to_snapshot_support",
                deprecated_for_removal=True,
                deprecated_reason="Redundant configuration option. Please use "
                                  "'run_revert_to_snapshot_tests' config "
                                  "option instead.",
                help="Defines extra spec that satisfies specific back end "
                     "capability called 'revert_to_snapshot_support' "
                     "and will be used for setting up custom share type. "
                     "Defaults to the value of run_revert_to_snapshot_tests."),
    cfg.StrOpt("capability_sg_consistent_snapshot_support",
               choices=["host", "pool", None],
               help="Backend capability to create consistent snapshots of "
                    "share group members. Will be used with creation "
                    "of new share group types as group spec."),
    cfg.BoolOpt("capability_thin_provisioned",
                default=False,
                help="Defines whether to create shares as thin provisioned, "
                     "adding the extra spec 'thin_provisioning' as 'True' for "
                     "setting up the custom share types. It may be useful to "
                     "run tempest with  back end storage systems without much "
                     "space. Take care enabling it, the manila scheduler "
                     "capability filter will request this capability in all "
                     "share types and the the capacity filter will allow "
                     "oversubscription."),
    cfg.StrOpt("share_network_id",
               default="",
               help="Some backend drivers requires share network "
                    "for share creation. Share network id, that will be "
                    "used for shares. If not set, it won't be used. Setting "
                    "this option to a valid share network ID will mean that "
                    "the value of create_networks_when_multitenancy_enabled "
                    "should be False."),
    cfg.StrOpt("alt_share_network_id",
               default="",
               help="Share network id, that will be used for shares"
                    " in alt tenant. If not set, it won't be used. Setting "
                    "this option to a valid share network ID will mean that "
                    "the value of create_networks_when_multitenancy_enabled "
                    "should be False."),
    cfg.StrOpt("admin_share_network_id",
               default="",
               help="Share network id, that will be used for shares"
                    " in admin tenant. If not set, it won't be used. Setting "
                    "this option to a valid share network ID will mean that "
                    "the value of create_networks_when_multitenancy_enabled "
                    "should be False."),
    cfg.BoolOpt("multi_backend",
                default=False,
                help="Runs Manila multi-backend tests."),
    cfg.ListOpt("backend_names",
                default=[],
                help="Names of share backends, that will be used with "
                     "multibackend tests. Tempest will use first two values."),
    cfg.IntOpt("share_creation_retry_number",
               default=0,
               help="Defines number of retries for share creation. "
                    "It is useful to avoid failures caused by unstable "
                    "environment."),
    cfg.IntOpt("build_interval",
               default=3,
               help="Time in seconds between share availability checks."),
    cfg.IntOpt("build_timeout",
               default=500,
               help="Timeout in seconds to wait for a share to become"
                    "available."),
    cfg.BoolOpt("suppress_errors_in_cleanup",
                default=False,
                help="Whether to suppress errors with clean up operation "
                     "or not. There are cases when we may want to skip "
                     "such errors and catch only test errors."),
    cfg.MultiOpt("security_service",
                 item_type=types.Dict(),
                 secret=True,
                 help="This option enables specifying security service "
                      "parameters needed to create security services "
                      "dynamically in order to run the tempest tests. "
                      "The configured security service must be reachable by "
                      "the project share networks created by the tests. So, "
                      "ideally project networks must be able to route to the "
                      "network where the pre-existing security services has "
                      "been deployed. The set of parameters that can be "
                      "configured is the same used in the security service "
                      "creation. You can repeat this option many times, and "
                      "each entry takes the standard dict config parameters: "
                      "security_service = "
                      "ss_type:<ldap, kerberos or active_directory>, "
                      "ss_dns_ip:value, ss_user:value, ss_password=value, "
                      "ss_domain:value, ss_server:value"),

    # Switching ON/OFF test suites filtered by features
    cfg.BoolOpt("run_quota_tests",
                default=True,
                help="Defines whether to run quota tests or not."),
    cfg.BoolOpt("run_extend_tests",
                default=True,
                help="Defines whether to run share extend tests or not. "
                     "Disable this feature if used driver doesn't "
                     "support it."),
    cfg.BoolOpt("run_shrink_tests",
                default=True,
                help="Defines whether to run share shrink tests or not. "
                     "Disable this feature if used driver doesn't "
                     "support it."),
    cfg.BoolOpt("run_snapshot_tests",
                default=True,
                help="Defines whether to run tests that use share snapshots "
                     "or not. Disable this feature if used driver doesn't "
                     "support it."),
    cfg.BoolOpt("run_revert_to_snapshot_tests",
                default=False,
                help="Defines whether to run tests that revert shares "
                     "to snapshots or not. Enable this feature if used "
                     "driver supports it."),
    cfg.BoolOpt("run_share_group_tests",
                default=True,
                deprecated_name="run_consistency_group_tests",
                help="Defines whether to run share group tests or not."),
    cfg.BoolOpt("run_replication_tests",
                default=False,
                help="Defines whether to run replication tests or not. "
                     "Enable this feature if the driver is configured "
                     "for replication."),
    cfg.BoolOpt("run_multiple_share_replicas_tests",
                default=True,
                help="Defines whether to run multiple replicas creation test "
                     "or not. Enable this if the driver can create more than "
                     "one replica for a share."),
    cfg.BoolOpt("run_host_assisted_migration_tests",
                deprecated_name="run_migration_tests",
                default=False,
                help="Enable or disable host-assisted migration tests."),
    cfg.BoolOpt("run_driver_assisted_migration_tests",
                deprecated_name="run_migration_tests",
                default=False,
                help="Enable or disable driver-assisted migration tests."),
    cfg.BoolOpt("run_migration_with_preserve_snapshots_tests",
                default=False,
                help="Enable or disable migration with "
                     "preserve_snapshots tests set to True."),
    cfg.BoolOpt("run_driver_assisted_backup_tests",
                default=False,
                help="Enable or disable share backup tests."),
    cfg.BoolOpt("run_manage_unmanage_tests",
                default=False,
                help="Defines whether to run manage/unmanage tests or not. "
                     "These test may leave orphaned resources, so be careful "
                     "enabling this opt."),
    cfg.BoolOpt("run_manage_unmanage_snapshot_tests",
                default=False,
                help="Defines whether to run manage/unmanage snapshot tests "
                     "or not. These tests may leave orphaned resources, so be "
                     "careful enabling this opt."),
    cfg.BoolOpt("run_mount_snapshot_tests",
                default=False,
                help="Enable or disable mountable snapshot tests."),
    cfg.BoolOpt("run_negative_migration_replica_tests",
                default=False,
                help="Enable or disable negative migration with replica "
                     "tests."),
    cfg.BoolOpt("run_positive_migration_replica_tests",
                default=True,
                help="Enable or disable positive migration with replica tests."
                ),
    cfg.BoolOpt("run_create_share_from_snapshot_in_another_pool_or_az_tests",
                default=False,
                help="Defines whether to run tests that create share from "
                     "snapshots in another pool or az. Enable this "
                     "option if the used driver supports it."),
    cfg.BoolOpt("run_share_server_migration_tests",
                default=False,
                help="Defines whether to run share servers migration tests. "
                     "Enable this option if the used driver supports it."),
    cfg.BoolOpt("run_share_server_multiple_subnet_tests",
                default=False,
                help="Defines whether to run the share server multiple "
                     "subnets tests. Enable this option if the used driver "
                     "supports it."),
    cfg.BoolOpt("run_network_allocation_update_tests",
                default=False,
                help="Defines whether to run the network allocation update "
                     "tests. Enable this option if the used driver "
                     "supports it."),

    cfg.StrOpt("image_with_share_tools",
               default="manila-service-image-master",
               help="Image name for vm booting with nfs/smb clients tool."),
    cfg.StrOpt("image_username",
               default="manila",
               help="Image username."),
    cfg.StrOpt("image_password",
               secret=True,
               help="Image password. Should be used for "
                    "'image_with_share_tools' without Nova Metadata support."),
    cfg.StrOpt("client_vm_flavor_ref",
               default="100",
               help="Flavor used for client vm in scenario tests."),
    cfg.IntOpt("migration_timeout",
               default=1500,
               help="Time to wait for share migration before "
                    "timing out (seconds)."),
    cfg.IntOpt("share_backup_timeout",
               default=1500,
               help="Time to wait for share backup before "
                    "timing out (seconds)."),
    cfg.IntOpt("share_server_migration_timeout",
               default=1500,
               help="Time to wait for share server migration before "
                    "timing out (seconds)."),
    cfg.StrOpt("default_share_type_name",
               help="Default share type name to use in tempest tests."),
    cfg.StrOpt("backend_replication_type",
               default='none',
               choices=['none', 'writable', 'readable', 'dr'],
               help="Specify the replication type supported by the backend."),
    cfg.IntOpt("share_size",
               default=1,
               help="Default size in GB for shares created by share tests."),
    cfg.IntOpt("additional_overflow_blocks",
               default=0,
               help="Additional blocks to be written "
                    "to share in scenario tests."),
    cfg.IntOpt("share_resize_sync_delay",
               default=0,
               help="Time to wait before the changes to the share size"
                    " are propagated to the storage system."),
    cfg.IntOpt("share_growth_size",
               default=1,
               help="The default increase in size sought by tests"
                    " when validating share resizing within scenario tests."),
    cfg.BoolOpt("run_ipv6_tests",
                default=False,
                help="Enable or disable running IPv6 NFS scenario tests. "
                     "These tests validate that IPv6 export locations work, "
                     "and that access can be provided to IPv6 clients. When "
                     "you do not specify a storage_network, the tests will "
                     "attempt to create an IPv6 subnet on the project network "
                     "they create for ping and SSH to the client test VM "
                     "where data path testing is performed."),
    cfg.StrOpt("dd_input_file",
               default="/dev/zero",
               help="The input file (if) in the dd command specifies the "
                    "source of data that dd will read and process, which can "
                    "be a device, a regular file, or even standard input "
                    "(stdin). dd copies, transforms, or performs actions on "
                    "this data based on provided options and then writes it "
                    "to an output file or device (of). When using /dev/zero "
                    "in storage systems with default compression, although "
                    "it generates highly compressible null bytes (zeros), "
                    "writing data from /dev/zero might not yield significant "
                    "space savings as these systems are already optimized for "
                    "efficient compression."),
    cfg.DictOpt("driver_assisted_backup_test_driver_options",
                default={'dummy': True},
                help="Share backup driver options specified as dict."),
]



# Copyright 2024 Cloudification GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
_MIN_SUPPORTED_MICROVERSION = '2.80'


class ShareBackupTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareBackupTest, cls).skip_checks()
        if not CONF.share.run_driver_assisted_backup_tests:
            raise cls.skipException("Share backup tests are disabled.")
        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    def setUp(self):
        super(ShareBackupTest, self).setUp()
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        share_type = self.create_share_type(extra_specs=extra_specs)
        share = self.create_share(self.shares_v2_client.share_protocol,
                                  share_type_id=share_type['id'])
        self.share_id = share["id"]

    @decorators.idempotent_id('12c36c97-faf4-4fec-9a9b-7cff0d2035cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_share_backup(self):
        backup = self.create_backup_wait_for_active(self.share_id)

        # Verify backup create API response, we use the configured max API
        # version to make this call
        expected_keys = ["id", "share_id", "status",
                         "availability_zone", "created_at", "updated_at",
                         "size", "progress", "restore_progress",
                         "name", "description"]
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.85'):
            expected_keys.append("backup_type")

        # Strict key check
        actual_backup = self.shares_v2_client.get_share_backup(
            backup['id'])['share_backup']
        actual_keys = actual_backup.keys()
        self.assertEqual(backup['id'], actual_backup['id'])
        self.assertEqual(set(expected_keys), set(actual_keys))

    @decorators.idempotent_id('34c36c97-faf4-4fec-9a9b-7cff0d2035cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_delete_share_backup(self):
        backup = self.create_backup_wait_for_active(
            self.share_id, cleanup=False)

        # Delete share backup
        self.shares_v2_client.delete_share_backup(backup['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            backup_id=backup['id'])
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_backup,
            backup['id'])

    @decorators.idempotent_id('56c36c97-faf4-4fec-9a9b-7cff0d2035cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_restore_share_backup(self):
        backup = self.create_backup_wait_for_active(self.share_id)

        # Restore share backup
        restore = self.shares_v2_client.restore_share_backup(
            backup['id'])['restore']
        waiters.wait_for_resource_status(
            self.shares_v2_client, backup['id'], 'available',
            resource_name='share_backup')

        self.assertEqual(restore['share_id'], self.share_id)

    @decorators.idempotent_id('78c36c97-faf4-4fec-9a9b-7cff0d2035cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_update_share_backup(self):
        backup = self.create_backup_wait_for_active(self.share_id)

        # Update share backup name
        backup_name2 = data_utils.rand_name('Backup')
        backup = self.shares_v2_client.update_share_backup(
            backup['id'], name=backup_name2)['share_backup']
        updated_backup = self.shares_v2_client.get_share_backup(
            backup['id'])['share_backup']
        self.assertEqual(backup_name2, updated_backup['name'])

    @decorators.idempotent_id('19c36c97-faf4-4fec-9a9b-7cff0d2045af')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_list_share_backups(self):
        self.create_backup_wait_for_active(self.share_id)
        backups = self.shares_v2_client.list_share_backups()
        self.assertEqual(1, len(backups))



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class AdminActionsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(AdminActionsNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client
        # create share type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id,
                                     client=cls.admin_client)
        cls.sh_instance = (
            cls.admin_client.get_instances_of_share(
                cls.share["id"])['share_instances'][0]
        )
        if CONF.share.run_snapshot_tests:
            cls.snapshot = cls.create_snapshot_wait_for_active(
                cls.share["id"], client=cls.admin_client)

    @decorators.idempotent_id('f730c395-a501-44cf-90d9-a3273771b895')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_share_state_to_unacceptable_state(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.reset_state,
                          self.share["id"], status="fake")

    @decorators.idempotent_id('3bfa9555-9c7e-45a2-b5bd-384329cb6fda')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_share_instance_state_to_unacceptable_state(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_client.reset_state,
            self.sh_instance["id"],
            s_type="share_instances",
            status="fake"
        )

    @decorators.idempotent_id('02e0d0d5-ac66-4d24-9aa7-568f75944a05')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_reset_snapshot_state_to_unacceptable_state(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.reset_state,
                          self.snapshot["id"],
                          s_type="snapshots",
                          status="fake")

    @decorators.idempotent_id('3b525c29-b657-493f-aa41-b17676a95fd2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_reset_share_state_with_member(self):
        # NOTE(gouthamr): The client used below is of a member from another
        # project. As a fix to bug #1901210, the server responds with
        # 404 instead of 403, but we'll test for one of the two codes since
        # the test could be running against a release without the fix.
        self.assertRaises((lib_exc.Forbidden, lib_exc.NotFound),
                          self.member_client.reset_state,
                          self.share["id"])

    @decorators.idempotent_id('d4abddba-1c20-49e1-85b1-5452f0faceb0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_reset_share_instance_state_with_member(self):
        # Even if member from another tenant, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.reset_state,
                          self.sh_instance["id"], s_type="share_instances")

    @decorators.idempotent_id('48dfb1ec-6db6-4022-8a41-2eb2883e0988')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_try_reset_snapshot_state_with_member(self):
        # Even if member from another tenant, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.reset_state,
                          self.snapshot["id"], s_type="snapshots")

    @decorators.idempotent_id('7cd0b48e-2815-4f8c-8718-3c071ff9701f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_force_delete_share_with_member(self):
        # If a non-admin tries to do force_delete, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.force_delete,
                          self.share["id"])

    @decorators.idempotent_id('257da3e0-9460-4d97-8a56-c86c0427cc64')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_force_delete_share_instance_with_member(self):
        # If a non-admin tries to do force_delete, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.force_delete,
                          self.sh_instance["id"], s_type="share_instances")

    @decorators.idempotent_id('c9a1894f-d58f-4885-86ba-736e9ab8428a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_try_force_delete_snapshot_with_member(self):
        # If a non-admin tries to do force_delete, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.force_delete,
                          self.snapshot["id"], s_type="snapshots")

    @decorators.idempotent_id('821da7c8-3501-44ba-9ffe-45f485a6e573')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_get_share_instance_with_member(self):
        # If a non-admin tries to get instance, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.get_share_instance,
                          self.sh_instance["id"])

    @decorators.idempotent_id('ab361521-adc9-4fe3-9699-a5ccc49b579b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_get_instances_of_share_with_member(self):
        # If a non-admin tries to list instances of given share, it should be
        # unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.get_instances_of_share,
                          self.share['id'])

    @decorators.idempotent_id('d662457c-2b84-4f13-aee7-5ffafe2552f1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_reset_task_state_invalid_state(self):
        self.assertRaises(
            lib_exc.BadRequest, self.admin_client.reset_task_state,
            self.share['id'], 'fake_state')


@ddt.ddt
class AdminActionsAPIOnlyNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(AdminActionsAPIOnlyNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client

    @decorators.idempotent_id('1c928920-1538-400a-ab28-c58dd75503c3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_share_instance_with_member(self):
        # If a non-admin tries to list instances, it should be unauthorized
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.list_share_instances)

    @decorators.idempotent_id('aba8638c-bfed-4c3e-994b-5309fcd912b2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_reset_task_state_share_not_found(self):
        self.assertRaises(
            lib_exc.NotFound, self.admin_client.reset_task_state,
            'fake_share', 'migration_error')

    @decorators.idempotent_id('e31d2d7b-7202-4699-9423-72f710e72181')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_force_delete_nonexistent_snapshot(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.force_delete,
                          "fake",
                          s_type="snapshots")

    @decorators.idempotent_id('dedca5c1-151d-40f7-bb7f-8913d51c05a9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_force_delete_nonexistent_share(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.force_delete, "fake")

    @decorators.idempotent_id('7cbfc035-12ea-4e2c-8da1-baf261e45f03')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_force_delete_nonexistent_share_instance(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.force_delete,
                          "fake",
                          s_type="share_instances")

    @decorators.idempotent_id('17e7eb3c-dbe6-4667-b838-663211365d44')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_reset_nonexistent_share_state(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.reset_state, "fake")

    @decorators.idempotent_id('26ce6f02-98eb-435a-9065-2e5bbcac87c5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_reset_nonexistent_share_instance_state(self):
        self.assertRaises(lib_exc.NotFound, self.admin_client.reset_state,
                          "fake", s_type="share_instances")

    @decorators.idempotent_id('7e07f684-b68f-4b0e-89cc-05d70a67dd69')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_reset_nonexistent_snapshot_state(self):
        self.assertRaises(lib_exc.NotFound, self.admin_client.reset_state,
                          "fake", s_type="snapshots")

    @decorators.idempotent_id('59b09ad2-d405-4762-a253-d7b7cf56f0a5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data('migrate_share', 'migration_complete', 'reset_task_state',
              'migration_get_progress', 'migration_cancel')
    def test_migration_API_invalid_microversion(self, method_name):
        if method_name == 'migrate_share':
            self.assertRaises(
                lib_exc.NotFound, getattr(self.shares_v2_client, method_name),
                'fake_share', 'fake_host', version='2.21')
        elif method_name == 'reset_task_state':
            self.assertRaises(
                lib_exc.NotFound, getattr(self.shares_v2_client, method_name),
                'fake_share', 'fake_task_state', version='2.21')
        else:
            self.assertRaises(
                lib_exc.NotFound, getattr(self.shares_v2_client, method_name),
                'fake_share', version='2.21')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Shares
STATUS_ERROR = 'error'
STATUS_AVAILABLE = 'available'
STATUS_ERROR_DELETING = 'error_deleting'
STATUS_MIGRATING = 'migrating'
STATUS_MANAGE_ERROR = 'manage_error'
STATUS_MIGRATING_TO = 'migrating_to'
STATUS_CREATING = 'creating'
STATUS_DELETING = 'deleting'
STATUS_SHRINKING = 'shrinking'
STATUS_SHRINKING_POSSIBLE_DATA_LOSS_ERROR = (
    'shrinking_possible_data_loss_error')
TEMPEST_MANILA_PREFIX = 'tempest-manila'

# Replication
REPLICATION_STYLE_READABLE = 'readable'
REPLICATION_STYLE_WRITABLE = 'writable'
REPLICATION_STYLE_DR = 'dr'
REPLICATION_TYPE_CHOICES = (
    REPLICATION_STYLE_READABLE,
    REPLICATION_STYLE_WRITABLE,
    REPLICATION_STYLE_DR,
)
REPLICATION_PROMOTION_CHOICES = (
    REPLICATION_STYLE_READABLE,
    REPLICATION_STYLE_DR,
)
REPLICATION_STATE_ACTIVE = 'active'
REPLICATION_STATE_IN_SYNC = 'in_sync'
REPLICATION_STATE_OUT_OF_SYNC = 'out_of_sync'
MIN_SHARE_REPLICATION_VERSION = '2.11'
SHARE_REPLICA_GRADUATION_VERSION = '2.56'
SHARE_REPLICA_SHARE_NET_PARAM_VERSION = '2.72'

# Access Rules
RULE_STATE_ACTIVE = 'active'
RULE_STATE_OUT_OF_SYNC = 'out_of_sync'
RULE_STATE_ERROR = 'error'

TASK_STATE_MIGRATION_STARTING = 'migration_starting'
TASK_STATE_MIGRATION_IN_PROGRESS = 'migration_in_progress'
TASK_STATE_MIGRATION_COMPLETING = 'migration_completing'
TASK_STATE_MIGRATION_SUCCESS = 'migration_success'
TASK_STATE_MIGRATION_ERROR = 'migration_error'
TASK_STATE_MIGRATION_CANCELLED = 'migration_cancelled'
TASK_STATE_MIGRATION_DRIVER_STARTING = 'migration_driver_starting'
TASK_STATE_MIGRATION_DRIVER_IN_PROGRESS = 'migration_driver_in_progress'
TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE = 'migration_driver_phase1_done'
TASK_STATE_DATA_COPYING_STARTING = 'data_copying_starting'
TASK_STATE_DATA_COPYING_IN_PROGRESS = 'data_copying_in_progress'
TASK_STATE_DATA_COPYING_COMPLETING = 'data_copying_completing'
TASK_STATE_DATA_COPYING_COMPLETED = 'data_copying_completed'
TASK_STATE_DATA_COPYING_CANCELLED = 'data_copying_cancelled'
TASK_STATE_DATA_COPYING_ERROR = 'data_copying_error'

# Revert to snapshot
REVERT_TO_SNAPSHOT_MICROVERSION = '2.27'
REVERT_TO_SNAPSHOT_SUPPORT = 'revert_to_snapshot_support'
STATUS_RESTORING = 'restoring'
STATUS_REVERTING = 'reverting'
STATUS_REVERTING_ERROR = 'reverting_error'

# Share groups
MIN_SHARE_GROUP_MICROVERSION = '2.31'
SHARE_GROUPS_GRADUATION_VERSION = '2.55'
SHARE_GROUP_SIMPLE_KEYS = {
    'id', 'name', 'links',
}
SHARE_GROUP_DETAIL_REQUIRED_KEYS = {
    'id', 'name', 'description', 'created_at', 'status', 'share_types',
    'project_id', 'host', 'links', 'share_group_type_id',
}
SHARE_GROUP_SNAPSHOT_SIMPLE_KEYS = {
    'id', 'name', 'links',
}
SHARE_GROUP_SNAPSHOT_DETAIL_REQUIRED_KEYS = {
    'id', 'name', 'description', 'created_at', 'status', 'project_id',
    'links', 'share_group_id', 'members',
}

SHARE_GROUP_TYPE_REQUIRED_KEYS = {
    'id', 'name', 'share_types', 'is_public', 'group_specs',
}

MIN_SHARE_ACCESS_METADATA_MICROVERSION = '2.45'

# Share servers
SERVER_STATE_ACTIVE = 'active'
SERVER_STATE_INACTIVE = 'inactive'
SERVER_STATE_CREATING = 'creating'
SERVER_STATE_DELETING = 'deleting'
SERVER_STATE_ERROR = 'error'
SERVER_STATE_MANAGE_ERROR = 'manage_error'
SERVER_STATE_MANAGE_STARTING = 'manage_starting'
SERVER_STATE_UNMANAGE_ERROR = 'unmanage_error'
SERVER_STATE_UNMANAGE_STARTING = 'unmanage_starting'
STATUS_SERVER_MIGRATING = 'server_migrating'
STATUS_SERVER_MIGRATING_TO = 'server_migrating_to'

# Share transfer
SHARE_TRANSFER_VERSION = "2.77"
